import datetime
import csv
import math
from collections import OrderedDict
import os
import matplotlib.pyplot as plt
import matplotlib.dates as mdates

def get_user_name_info(gid):
    d_user_name_info = dict()
    with open('passwd', 'r') as f:
        for line in f.readlines():
            l = line.strip().split(':')

            username = l[0]
            uid = int(l[2])
            tmp_gid = int(l[3])
            user_info = l[4]
            home_dir = l[5]


            if tmp_gid == gid:
                d_user_name_info[username] = user_info

    return d_user_name_info

def calculate_seconds_size_speed(filename):
	# Step 1: accumualte seconds and size
	d_user_seconds_size = {}    # username : [seconds, acculated_size, previous_size]
	with open(filename, 'r', encoding='utf-8') as f:
		for row in csv.reader(f, delimiter=','):
			username = row[1]
			size = int(row[-1])

			# if username != '202021121016':
			# 	continue

			if username not in d_user_seconds_size:
				# the first record
				d_user_seconds_size[username] = [0, 0, size]
			else:
				d_user_seconds_size[username] = [d_user_seconds_size[username][0] + 1,
				                                 d_user_seconds_size[username][1] + abs(size - d_user_seconds_size[username][2]),
				                                 size]


			# print(row)
			# print(username, d_user_seconds_size[username][0], d_user_seconds_size[username][1])
	# Step 2: calulate speed and sorted
	d_user_seconds_size_speed = {}
	for username, t in d_user_seconds_size.items():
		if t[0] == 0:
			d_user_seconds_size_speed[username] = [t[0], t[1], 0]
		else:
			d_user_seconds_size_speed[username] = [t[0], t[1], t[1] / t[0]]

	d_user_seconds_size_speed_sorted = OrderedDict(sorted(d_user_seconds_size_speed.items(), key=lambda x: x[1][-1], reverse=True))

	return d_user_seconds_size_speed_sorted

def save_userinfo_seconds_size_speed(outfile, d_user_seconds_size_speed, d_user_name_info):
	lists = [['class_id', 'student_id', 'chinese_name', 'seconds', 'acc_size', 'speed']]

	for username, t in d_user_seconds_size_speed.items():
		if username == '201761000037':
			username_os = 'qiankun'
		else:
			username_os = 'u{}'.format(username)

		class_id, student_id, chinese_name = d_user_name_info[username_os].split('-')
		seconds, acc_size, speed = t

		lists.append([class_id, student_id, chinese_name, seconds, acc_size, round(speed, 2)])

	# write to file
	with open(outfile, 'w', newline='') as f:
		writer = csv.writer(f)
		writer.writerows(lists)

def draw_file_changes(input_file, output_dir, d_user_name_info, d_user_seconds_size_speed):
	d_user_time_size = {}
	with open(input_file, 'r', encoding='utf-8') as f: # 2022-05-30 13:58:37,201761000037,reader_writer_pv.c,769

		for row in csv.reader(f, delimiter=','):
			username = row[1]
			size = int(row[-1])
			dt = datetime.datetime.strptime(row[0], '%Y-%m-%d %H:%M:%S')

			# if username != '202021121054': #'202021121018':  #201761000037
			# 	continue

			if username not in d_user_time_size:
				d_user_time_size[username] = [(dt, size)]
			else:
				d_user_time_size[username].append((dt, size))

	# draw a graph
	for username, lists in d_user_time_size.items():
		# use user_info as output filename
		if username == '201761000037':
			username_os = 'qiankun'
		else:
			username_os = 'u{}'.format(username)

		user_info = d_user_name_info[username_os]
		output_filename = '{}/{}.jpg'.format(output_dir, user_info)

		# draw a graph
		fig, ax = plt.subplots()

		columns = list(zip(*lists))
		# plt.plot(columns[0], columns[1], 'r-x', linewidth=0.5, label='20202112****') # label=user_info
		plt.plot(columns[0], columns[1], 'r-x', linewidth=0.5, label=user_info)  # label=user_info

		seconds, size, speed = d_user_seconds_size_speed[username]
		title = '文件变化次数={}   累计文件大小={}   速度={}'.format(seconds, size, round(speed, 2))

		plt.title(title)
		plt.xlabel('时间')
		plt.ylabel('文件大小（单位：字节）')
		plt.grid(True)
		plt.legend(loc='best')


		start_dt = datetime.datetime.strptime('2022-05-30 13:50', '%Y-%m-%d %H:%M')
		end_dt = datetime.datetime.strptime('2022-05-30 15:40', '%Y-%m-%d %H:%M')
		plt.xlim(start_dt, end_dt)

		# date format
		date_format = mdates.DateFormatter('%H:%M')
		ax.xaxis.set_major_formatter(date_format)

		#plt.xticks(columns[0]) #rotation='vertical'
		fig.tight_layout()

		plt.savefig(output_filename)

		# plt.show()


def main():
	d_user_name_info = get_user_name_info(201112)

	filename = 'file_changes_exp4_reader_writer_pv_2022.csv'  # 2022-05-30 13:58:37,201761000037,reader_writer_pv.c,769
	d_user_seconds_size_speed = calculate_seconds_size_speed(filename)
	#
	# # save to file
	# outfile = 'stats_file_changes_exp3.csv'
	# save_userinfo_seconds_size_speed(outfile, d_user_seconds_size_speed, d_user_name_info)

	# draw graph
	output_dir = 'images_file_changes'
	draw_file_changes(filename, output_dir, d_user_name_info, d_user_seconds_size_speed)



	return


	pass

if __name__ == '__main__':
	main()